<?php

namespace Drupal\mymodule\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\AlertCommand;
use Drupal\Core\Ajax\CssCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Url;
use Drupal\redirect\Entity\Redirect;

class AjaxForm extends FormBase {
  public function getFormId() {
    return 'mymodule_ajax_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['name'] = array(
      '#type' => 'textfield',
      '#title' => 'Name',
      '#required' => TRUE,
      '#prefix' => '',
      '#suffix' => '<div class="name-validation-message"></div>',
    );

    $form['email'] = array(
      '#type' => 'email',
      '#title' => 'Email',
      '#required' => TRUE,
      '#prefix' => '',
      '#suffix' => '<div class="email-validation-message"></div>',
      '#ajax' => [
        'callback' => [$this, 'validateEmailCallbackAjax'],
        'event' => 'change',
        'wrapper' => 'edit-email',
      ],
    );

    $form['url'] = array(
      '#type' => 'textfield',
      '#title' => 'URL',
      '#prefix' => '',
      '#suffix' => '<div class="url-validation-message"></div>',
      '#ajax' => [
        'callback' => [$this, 'validateUrlCallbackAjax'],
        'event' => 'change',
        'wrapper' => 'edit-email',
      ],
      '#states' => array(
        'visible' => array(
          ':input[name="site"]' => ['value' => 'hasSite'],
        ),
      ),
    );

    $form['site'] = array(
      '#type' => 'radios',
      '#description' => t('Select'),
      '#attributes' => [
        'name' => 'site'
      ],
      '#default_value' => 'noSite',
      '#required' => TRUE,
      '#options' => array(
        'hasSite' => t('У меня есть сайт'),
        'noSite' => t('У меня нету сайта'),
      )
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Click',
      '#ajax' => [
        'callback' => [$this, 'alert'],
        'event' => 'click',
      ],
    );
    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    $url = $form_state->getValue('url');
    $email = $form_state->getValue('email');
    $site = $form_state->getValue('site');
    if ($site == 'hasSite') {
      if (!$this->checkUrl($url) or empty($url)) {
        $form_state->setErrorByName('url', 'Неверно введен url');
      }
    }
    if (!$this->checkEmail($email)) {
      $form_state->setErrorByName('email', 'Неверно введен email');
    }
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    #$url = Url::fromRoute('mymodule.batch_form');
    #$form_state->setRedirectUrl();
    #$form_state->setRedirect();

    $query = \Drupal::database()->insert('AAA_custom_table');
    $query->fields([
      'name' => $form_state->getValue('name'),
      'email' => $form_state->getValue('email'),
      'url' => $form_state->getValue('url'),
    ]);
    $query->execute();
    $form_state->setRedirect('<front>');
  }

  public function getResponse(array &$form, FormStateInterface $form_state, $field_name) {
    $response = new AjaxResponse();
    $val = $field_name == 'email' ? $val = $form_state->getValue('email') : $val = $form_state->getValue('url');
    $check = $field_name == 'email' ? $check = 'checkEmail' : $check = 'checkUrl';

    if (!$this->$check($val)) {
      $response->addCommand(new HtmlCommand('.email-validation-message', "Введен не корректный $field_name"));
      $response->addCommand(new CssCommand('.email-validation-message', ['color' => 'red']));
      $response->addCommand(new CssCommand('#edit-email', ['border' => '1px solid red']));
    } else {
      $response->addCommand(new HtmlCommand('.email-validation-message', ''));
      $response->addCommand(new CssCommand('#edit-email', ['border' => '']));
    }
    return $response;
  }

  public function validateEmailCallbackAjax(array &$form, FormStateInterface $form_state) {
    $field_name = 'email';
    $response = $this->getResponse($form, $form_state, $field_name);
    return $response;
  }

  public function validateUrlCallbackAjax(array &$form, FormStateInterface $form_state) {
    $field_name = 'url';
    $response = $this->getResponse($form, $form_state, $field_name);
    return $response;
  }

  public function checkUrl($url) {
    $regexp = '/^(https|http):\/\//';
    $checkHttp = (bool)preg_match($regexp, $url);
    if ($checkHttp) {
      $url[4] == 's' ? $httpOrHttps = 8 : $httpOrHttps = 7;
      $checkHttp ? $url = substr($url, $httpOrHttps) : '';
    }

    $regexp = '/^www./';
    $checkWww = ((bool)preg_match($regexp, $url));
    if ($checkWww) {
      $url = substr($url, 4);
    }

    $regexp = '/^([a-z0-9]*.[rр][uф])/';
    $checkUrl = (bool)preg_match($regexp, $url);

    $url = explode('.',$url);
    $url = $url[0] . '.' . $url[1];
    if ($checkUrl) {
      return true;
    } else {
      return false;
    }
  }

  public function checkEmail($email) {
    $email_valid = filter_var($email, FILTER_VALIDATE_EMAIL);
    if ($email_valid and !empty($email)) {
      return true;
    } else {
      return false;
    }
  }
  public function alert(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $url = $form_state->getValue('url');
    $email = $form_state->getValue('email');
    $site = $form_state->getValue('site');
    $commandRedirect = new RedirectCommand('/home');

    if ($site == 'hasSite') {
      if (!$this->checkUrl($url) or empty($url)) {
        $form_state->setErrorByName('url', 'Неверное введен url');
      }
    }

    if (!$this->checkEmail($email)) {
      $form_state->setErrorByName('email', 'Неверное введен email');
    }
    $response->addCommand(new AlertCommand('Спасибо за заполнение'));
    $response->addCommand($commandRedirect);
    return $response;
  }
}
